#
# Include this file in your shell script by using:
#         #!/bin/bash
#         . ./wvtest.sh
#

# we don't quote $TEXT in case it contains newlines; newlines
# aren't allowed in test output.  However, we set -f so that
# at least shell glob characters aren't processed.
_wvtextclean()
{
	( set -f; echo $* )
}


if [ -n "$BASH_VERSION" ]; then
        trap 'echo "! ${BASH_SOURCE[0]}:${LINENO} ${BASH_COMMAND} FAILED"' ERR

	_wvfind_caller()
	{
		LVL=$1
		WVCALLER_FILE=${BASH_SOURCE[2]}
		WVCALLER_LINE=${BASH_LINENO[1]}
	}
else
	_wvfind_caller()
	{
		LVL=$1
		WVCALLER_FILE="unknown"
		WVCALLER_LINE=0
	}
fi


_wvcheck()
{
	CODE="$1"
	TEXT=$(_wvtextclean "$2")
	OK=ok
	if [ "$CODE" -ne 0 ]; then
		OK=FAILED
	fi
	echo "! $WVCALLER_FILE:$WVCALLER_LINE  $TEXT  $OK" >&2
	if [ "$CODE" -ne 0 ]; then
		exit $CODE
	else
		return 0
	fi
}


WVPASS()
{
	TEXT="$*"

	_wvfind_caller
	if "$@"; then
		_wvcheck 0 "$TEXT"
		return 0
	else
		_wvcheck 1 "$TEXT"
		# NOTREACHED
		return 1
	fi
}


WVFAIL()
{
	TEXT="$*"

	_wvfind_caller
	if "$@"; then
		_wvcheck 1 "NOT($TEXT)"
		# NOTREACHED
		return 1
	else
		_wvcheck 0 "NOT($TEXT)"
		return 0
	fi
}


_wvgetrv()
{
	( "$@" >&2 )
	echo -n $?
}


WVPASSEQ()
{
	_wvfind_caller
	_wvcheck $(_wvgetrv [ "$#" -eq 2 ]) "exactly 2 arguments"
	echo "Comparing:" >&2
	echo "$1" >&2
	echo "--" >&2
	echo "$2" >&2
	_wvcheck $(_wvgetrv [ "$1" = "$2" ]) "'$1' = '$2'"
}


WVPASSNE()
{
	_wvfind_caller
	_wvcheck $(_wvgetrv [ "$#" -eq 2 ]) "exactly 2 arguments"
	echo "Comparing:" >&2
	echo "$1" >&2
	echo "--" >&2
	echo "$2" >&2
	_wvcheck $(_wvgetrv [ "$1" != "$2" ]) "'$1' != '$2'"
}


WVPASSRC()
{
	RC=$?
	_wvfind_caller
	_wvcheck $(_wvgetrv [ $RC -eq 0 ]) "return code($RC) == 0"
}


WVFAILRC()
{
	RC=$?
	_wvfind_caller
	_wvcheck $(_wvgetrv [ $RC -ne 0 ]) "return code($RC) != 0"
}


WVSTART()
{
	echo >&2
	_wvfind_caller
	echo "Testing \"$* ($OMK_RULES)\" in $WVCALLER_FILE:" >&2
	cd "$WV_BASE_DIR"
	local dir=test.$(echo $* | tr -s '[[:blank:]]' _ | tr "A-Z" "a-z")
	rm -rf "$dir"
	mkdir -p "$dir"
	WVPASS cd "$WV_BASE_DIR/$dir"
	WVPASS cp "$OMK_TESTSROOT"/../rules/$OMK_RULES/* .
	WVPASS cp "$OMK_TESTSROOT"/../snippets/Makefile.rules.$OMK_RULES Makefile.rules
	WVPASS cp "$OMK_TESTSROOT/../leaf-makefile/Makefile" .
}

omkize()
{
    WVPASS cp "$OMK_TESTSROOT/../leaf-makefile/Makefile" .
    for i in `find -L . -mindepth 2 -name Makefile.omk`; do
	d=`dirname ${i}`
	WVPASS cp Makefile "${d}/Makefile"
    done
}

needs_valid_CC()
{
    CC=$(make omk-get-var VAR=CC|sed -ne '/^CC=/ s///p')
    if ! which ${CC%% *} > /dev/null; then
	echo "Warning: '$CC' not found - skipping the test"
	exit 0
    fi

    if [[ $OMK_RULES = sysless-keil16x ]] && ! which cygpath; then
	echo "Warning: cygpath command not found - skipping the test"
	exit 0
    fi

    if [[ $OMK_RULES = sysless-keil51 ]] && ! which l51; then
	echo "Warning: l51 command not found - skipping the test"
	exit 0
    fi
}

OMK_TESTSROOT=${OMK_TESTSROOT:-$(readlink --canonicalize $(dirname $0))}
OMK_RULES=${OMK_RULES:-linux}	# When run manually, we use linux rules

if [[ -f "${OMK_TESTSROOT}/${OMK_RULES}.env" ]]; then
    WVPASS . "${OMK_TESTSROOT}/${OMK_RULES}.env"
fi

case $OMK_RULES in
    rtems)
	[[ -z "$RTEMS_MAKEFILE_PATH" ]] && exit 0
	;;
    vxworks)
	[[ -z "$WRENV" ]] && exit 0
	;;
esac

echo "Testing \"$OMK_RULES rules\" in $(basename $0):" >&2
testdir="tmp/$(basename $0 .sh).$OMK_RULES"
[[ -d "$testdir" ]] && WVPASS rm -rf "$testdir"
WVPASS mkdir -p "$testdir"
WVPASS cd "$testdir"
WVPASS cp "$OMK_TESTSROOT"/../rules/$OMK_RULES/* .
WVPASS cp "$OMK_TESTSROOT"/../snippets/Makefile.rules.$OMK_RULES Makefile.rules
WVPASS cp "$OMK_TESTSROOT/../leaf-makefile/Makefile" .

WV_BASE_DIR="$PWD"
export MAKEFLAGS="--include-dir=$OMK_TESTSROOT/../snippets"
export LC_ALL=C
